'/*--------------------------------------------------------------------------------------+
'|
'|    $RCSfile: manageExceptionLogs.vbs,v $
'|   $Revision: 1.1.2.3 $
'|       $Date: 2004/08/05 19:16:15 $
'|
'|  $Copyright: (c) 2004 Bentley Systems, Incorporated. All rights reserved. $
'|
'+--------------------------------------------------------------------------------------*/
'
'
' When an exception occurs, MicroStation appends the details of the exception 
' to the Exception.log file and rewrites the binary file MiniDump.dat file.  A
' V8.5 exception can generate approximately 240K in the Exception.log file and
' 80K to MiniDump.dmp[1].  After generating these log files, MicroStation
' attempts to launch an external process to operate on these files.
'
' In 8.5, $(MS_ProcessExceptionLog) is examined and parsed for both a 
' "window display control" (see SW_xxxx constants in the Microsoft documentation of "ShowWindow")
' value and an executable with optional arguments.
'
' For example, set:
'	MS_ProcessExceptionLog
' to
'	0   "$(windir)\system32\wscript.exe"  $(MSDIR)manageExceptionLogs.vbs
' in mslocal.cfg
'
' This indicates that wscript should run manageExceptionLogs.vbs in a hidden window.
'
' manageExceptionLogs.vbs has following steps:
'  1) Delete "$(MS_TEMP)ExceptionHistory-20" (or whatever "g_maxLogsToKeep" is set in the script)
'  2) Shift any existing subdirectories to the next higher value. For example
'       move  $(MS_TEMP)ExceptionHistory-19  $(MS_TEMP)ExceptionHistory-20
'       move  18->19, 17->18, ... 1->2
'  3) Create $(MS_TEMP)ExceptionHistory-1\
'  4) Move the recently created Exception.log and MiniDUmp.dmp into $(MS_TEMP)ExceptionHistory-1\
'  5) Optionally send e-mail to the CAD Manager with the name of the computer where the exception
'       occurred.  This can be particularly useful for initial deployment of MicroStation.
'
'-------
'[1] Since MiniDump.dmp is a snapshot of MicroStation's process memory the size of the file can be
'    10's of megabytes or more based on the setting of $(MS_MiniDumpType) and the amount of virtual
'    memory consumed by MicroStation at the time of the fault.
'
'+--------------------------------------------------------------------------------------*

Option Explicit

' CHANGE THE NEXT 4 VARAIBLES TO MATCH YOUR ENVIRONMENT
Const    g_maxLogsToKeep    = 20                                ' Number of exception logs to keep in rotating named directories

'    These are for optionally sending e-mail when an exception occurs.
Const    g_emailHostName        = ""                            ' Exchange Server or SMTP server's DNS name or IP address (ex:mailhost.xyzzy.com)
Const    g_emailCADAdministrator= ""         ' Address of the person to notify
Const    g_emailUserDomain      = ""                 ' An optional suffix (or anything else) to append to the sender's address


'+--------------------------------------------------------------------------------------*


' Exit codes returned via %errorlevel%.  Pass an an argument to "WScript.Quit"
Const exitcodeSuccess           = 0
Const exitcodeInvalidSyntax	= 1
Const exitcodeError		= 2
Const exitcodeUnexpected	= 3
Const exitcodeWSHVersionError	= 4
Const exitcodeInvalidHost	= 5
Const exitcodeInvalidArgument	= 6


' Global Variables
Dim  g_oFSO       'As Scripting.FileSystemObject         ' File System Object
Dim  g_oWSH       'As WScript.Shell                      ' Script arguments, etc
Dim  g_oWSHEnv    'As WScript.Shell.Environment          ' Process' Environment variables


'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'*   Invoke the main routine
'+---------------+---------------+---------------+---------------+---------------+------*/
Main



'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'+---------------+---------------+---------------+---------------+---------------+------*/
Sub Main

    On Error Resume Next                                        ' default error trapping mechanism

    Set g_oFSO    = CreateObject("Scripting.FileSystemObject") : CheckForError
    Set g_oWSH    = CreateObject("WScript.Shell") : CheckForError
    Set g_oWSHEnv = g_oWSH.Environment("PROCESS")

    RotateExceptionLogDirectories
    
    if 0 <> strlen (g_emailHostName) then                       ' Set "g_emailHostName" above if you want to send e-mail to someone about this crash
        SendEmail
    End If

    
    ' Exit this script cleanly
    Set g_oWSHEnv = Nothing
    Set g_oWSH    = Nothing
    Set g_oFSO    = Nothing

    WScript.Quit exitcodeSuccess
End Sub

'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'+---------------+---------------+---------------+---------------+---------------+------*/
Sub RotateExceptionLogDirectories

    On Error Resume Next                                        ' default error trapping mechanism

    dim  strExceptionLogFullPath
    strExceptionLogFullPath = WScript.Arguments.UnNamed.Item(0)

    dim  strMicroStationTempPath
    strMicroStationTempPath = g_oFSO.GetParentFolderName(strExceptionLogFullPath)

    dim  strHistoryPath
    strHistoryPath = strMicroStationTempPath + "\ExceptionHistory-"

    ' Delete the oldest directory. The directories contain both exception.log and minidump.dmp files
    dim strOldestPath
    strOldestPath = strHistoryPath & g_maxLogsToKeep
    If g_oFSO.FolderExists(strOldestPath) Then
        g_oFSO.DeleteFolder strOldestPath, True 
    End If

    ' Loop through directories renaming "MicroStation\temp\ExceptionHistory-19"  to  "MicroStation\temp\ExceptionHistory-20", 18->19, ... 1->2
    dim  i
    dim  strOldPath, strNewPath
    For i = g_maxLogsToKeep  To  2  Step -1
        strOldPath = strHistoryPath & (i - 1)
        strNewPath = strHistoryPath & (i    )
        If g_oFSO.FolderExists(strOldPath) Then 
	    g_oFSO.MoveFolder strOldPath, strNewPath	: CheckForError
        End If
    Next

    ' Move the newest exception.log and minidump.dmp to "MicroStation\temp\ExceptionHistory-1"
    strNewPath = strHistoryPath + "1"
    g_oFSO.CreateFolder strNewPath
    g_oFSO.MoveFile  strMicroStationTempPath+"\Exception.log",  strNewPath+"\Exception.log" 	: CheckForError
    g_oFSO.MoveFile  strMicroStationTempPath+"\MiniDump.dmp",   strNewPath+"\MiniDump.dmp"      : CheckForError

    ' You could compress the exception.log and minidump.dmp files here or move them to a server for later analysis
    
End Sub


'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'+---------------+---------------+---------------+---------------+---------------+------*/
Sub SendEmail
    Dim iMsg, iConf, Flds
        
    Const cdoSendUsingPort =  2         ' Send the message using the network (SMTP over the network). 
    Const emailHostPort    = 25         ' Exchange Server listening port. Default is 25
    Const emailHostTimeout = 10         ' Exchange Server timeout in seconds
    
    On Error Resume Next                                        ' default error trapping mechanism
    
    ' initiatlize variables
    Set iMsg  = CreateObject("CDO.Message")       : CheckForError
    Set iConf = CreateObject("CDO.Configuration") : CheckForError
    Set Flds  = iConf.Fields 
    
    With Flds
        .Item("http://schemas.microsoft.com/cdo/configuration/sendusing")	      = cdoSendUsingPort
        .Item("http://schemas.microsoft.com/cdo/configuration/smtpserver")	      = g_emailHostName
        .Item("http://schemas.microsoft.com/cdo/configuration/smtpserverport")        = emailHostPort
        .Item("http://schemas.microsoft.com/cdo/configuration/smtpconnectiontimeout") = emailHostTimeout
        .Update
    End With
    
    ' COnstruct the message
    With iMsg
        Set .Configuration = iConf
        .To       = g_emailCADAdministrator                                     ' These fields..
        .From     = g_oWSHEnv("USERNAME") + g_emailUserDomain                   '..should be modifed..
        .Subject  = "MicroStation recorded an exception."                       '..to suit your needs
	.TextBody = "On computer " & g_oWSHEnv("COMPUTERNAME")                  ' This could be some other.. 
        '.AddAttachment(strLogFile)                                             '..site specific information here

        .Send : CheckForError     
    End With
    
    ' clean up
    Set iMsg = Nothing
    Set iConf = Nothing
    Set Flds = Nothing
End Sub


'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'+---------------+---------------+---------------+---------------+---------------+------*/
Sub CheckForError
    If Err = 0 Then Exit Sub                ' do not set the default error trapping mechanism -- it will reset the "err" object
    LogEvent (Err.Source & vbNewLine & Hex(Err.Number) & ": " & Err.Description)
    WScript.Quit exitcodeError
End Sub


'/*---------------------------------------------------------------------------------**//**
'* @bsimethod                                                    Bentley         02/04
'+---------------+---------------+---------------+---------------+---------------+------*/
Sub LogEvent (strMessage)
    ' EventLog event types
    Const eventtypeSuccess	= 0
    Const eventtypeError	= 1
    Const eventtypeWarning	= 2
    Const eventtypeInformation	= 4

    g_oWSH.LogEvent eventtypeInformation, WScript.ScriptName & ":" & vbNewLine & strMessage
End Sub
